/*
Copyright (c) 2003-2013, CKSource - Frederico Knabben. All rights reserved.
For licensing, see LICENSE.html or http://ckeditor.com/license
*/

/**
 * @file Plugin for inserting images from Drupal media module
 */
( function() {
  CKEDITOR.plugins.add( 'media',
  {
    // Wrap Drupal plugin in a proxy plugin.
    init: function(editor)
    {
      var pluginCommand = {
        exec: function (editor) {
          var data = {
            format: 'html',
            node: null,
            content: ''
          };
          var selection = editor.getSelection();

          if (selection) {
            data.node = selection.getSelectedElement();
            if (data.node) {
              data.node = data.node.$;
            }
            if (selection.getType() == CKEDITOR.SELECTION_TEXT) {
              if (CKEDITOR.env.ie) {
                data.content = selection.getNative().createRange().text;
              }

              else {
                data.content = selection.getNative().toString();
              }
            }
            else if (data.node) {
              // content is supposed to contain the "outerHTML".
              data.content = data.node.parentNode.innerHTML;
            }
          }
          Drupal.settings.ckeditor.plugins['media'].invoke(data, Drupal.settings.ckeditor.plugins['media'], editor.name);
        }
      };
      editor.addCommand( 'media', pluginCommand );

      editor.ui.addButton( 'Media',
      {
        label: 'Add media',
        command: 'media',
        icon: this.path + 'images/icon.gif'
      });

      // Because the media comment wrapper don't work well for CKEditor we
      // replace them by using a custom mediawrapper element.
      // Instead having
      // <!--MEDIA-WRAPPER-START-1--><img /><!--MEDIA-WRAPPER-END-1-->
      // We wrap the placeholder with
      // <mediawrapper data="1"><img /></mediawrapper>
      // That way we can deal better with selections - see selectionChange.
      CKEDITOR.dtd['mediawrapper'] = CKEDITOR.dtd;
      CKEDITOR.dtd.$blockLimit['mediawrapper'] = 1;
      CKEDITOR.dtd.$inline['mediawrapper'] = 1;
      CKEDITOR.dtd.$nonEditable['mediawrapper'] = 1;
      if (parseInt(CKEDITOR.version) > 3) {
        // Register allowed tag for advanced filtering.
        editor.filter.allow( 'mediawrapper[!data]', 'MediaWrapper' );
        // Objects should be selected as a whole in the editor.
        CKEDITOR.dtd.$object['mediawrapper'] = 1;
      }
      function prepareDataForWysiwygMode(data) {
        data = Drupal.media.filter.replaceTokenWithPlaceholder(data);
        data = data.replace(/<!--MEDIA-WRAPPER-START-(\d+)-->(.*?)<!--MEDIA-WRAPPER-END-\d+-->/gi, '<mediawrapper data="$1">$2</mediawrapper>');
        return data;
      }
      function prepareDataForSourceMode(data) {
        data = data.replace(/<mediawrapper data="(\d+)">(.*?)<\/mediawrapper>/gi, '<!--MEDIA-WRAPPER-START-$1-->$2<!--MEDIA-WRAPPER-END-$1-->');
        data = Drupal.media.filter.replacePlaceholderWithToken(data);
        return data;
      }

      // Ensure tokens instead the html element is saved.
      editor.on('getData', function( event ) {
        event.data.dataValue = prepareDataForSourceMode(event.data.dataValue);
      });

      // Ensure the tokens are replaced by placeholders while editing.
      if (parseInt(CKEDITOR.version) > 3) {
        // CKEditor 4.x behaviour.
        editor.on('setData', function( event ) {
          event.data.dataValue = prepareDataForWysiwygMode(event.data.dataValue);
        });
      }
      else {
        // CKEditor 3.6.x behaviour.
        editor.on( 'beforeSetMode', function( event, data ) {
          event.removeListener();
          var wysiwyg = editor._.modes[ 'wysiwyg' ];
          var source = editor._.modes[ 'source' ];
          wysiwyg.loadData = CKEDITOR.tools.override( wysiwyg.loadData, function( org )
          {
            return function( data ) {
              return ( org.call( this, prepareDataForWysiwygMode(data)) );
            };
          } );
          source.loadData = CKEDITOR.tools.override( source.loadData, function( org )
          {
            return function( data ) {
              return ( org.call( this, prepareDataForSourceMode(data) ) );
            };
          } );
        });
      }

      // Ensure our enclosing wrappers are always included in the selection.
      editor.on('selectionChange', function( event ) {
        var ranges = editor.getSelection().getRanges().createIterator();
        var newRanges = [];
        var currRange;
        while(currRange = ranges.getNextRange()){
          var commonAncestor = currRange.getCommonAncestor(false);
          if (commonAncestor.getName() == 'mediawrapper') {
            var range = new CKEDITOR.dom.range( editor.document );
            if (currRange.collapsed === true) {
              // Don't allow selection within the wrapper element.
              if (currRange.startOffset == 0) {
                // While v3 plays nice with setting start and end to avoid
                // editing within the media wrapper element, v4 ignores that.
                // Thus we try to move the cursor further away.
                if (parseInt(CKEDITOR.version) > 3) {
                  range.setStart(commonAncestor.getPrevious());
                  range.setEnd(commonAncestor.getPrevious());
                }
                else {
                  range.setStartBefore(commonAncestor);
                }
              }
              else {
                // While v3 plays nice with setting start and end to avoid
                // editing within the media wrapper element, v4 ignores that.
                // Thus we try to move the cursor further away.
                if (parseInt(CKEDITOR.version) > 3) {
                  range.setStart(commonAncestor.getNext(), 1);
                  range.setEnd(commonAncestor.getNext(), 1);
                }
                else {
                  range.setStartAfter(commonAncestor);
                }
              }
            }
            else {
              // Always select the whole wrapper element.
              range.setStartBefore(commonAncestor);
              range.setEndAfter(commonAncestor);
            }
            newRanges.push(range);
          }
        }
        if (newRanges.length) {
          editor.getSelection().selectRanges(newRanges);
        }
      });
    }
  });

} )();


